
/**
 *  Example native extension interface implementation for HelloWorld_iphone.js
 */
var GreetingsInterface = require('/shared/mbanking/ext/interfaces/Greetings');
var Config = require('/shared/mbanking/modules/config.mod');
var HelloWorldModule = require('com.mfoundry.mb.helloworld');
var Util = require('/shared/mbanking/utils/util');
var AppActivity = require('/shared/mbanking/modules/appactivity.mod');

var NATIVE_EXTENSION_APP_NAME = "Hello World Native Extension";
var NATIVE_EXTENSION_PROPERTY_SAMPLE = "com.mfoundry.mb.helloworld.sample";
var NATIVE_EXTENSION_ERROR_CODE = "com.mfoundry.mb.helloworld.error_code";

var GET_GREETING_EVENT = 'com.mfoundry.mb.helloworld:get_greeting';
var GET_GREETING_RESP_EVENT = 'com.mfoundry.mb.helloworld:get_greeting_resp';

var SET_GREETING_EVENT = 'com.mfoundry.mb.helloworld:set_greeting';
var SET_GREETING_RESP_EVENT = 'com.mfoundry.mb.helloworld:set_greeting_resp';

function HelloWorld(configuration) {
    this.configuration = configuration;
	_setupAppEventHandlers.call(this);
}

HelloWorld.prototype = GreetingsInterface.createPrototype();

HelloWorld.prototype.getGreeting = function() {
    try {
    	var errorProp = Config.P(NATIVE_EXTENSION_ERROR_CODE);
    	if (errorProp && errorProp !== '0') {
    		AppActivity.logActivity("com.mfoundry.mb.helloworld", "GreetingsSimulatedError", {"location" : "getGreeting"}, errorProp);
    	}
    	return HelloWorldModule.exampleProp;
    } catch (e) {
    	Util.error('Problem getting greeting: ' + e.message);
    	throw e;
    }
}


HelloWorld.prototype.setGreeting = function(message) {

	try {
		HelloWorldModule.exampleProp = message;
	} catch (e) {
		Util.error('Problem setting greeting: ' + e.message);
		throw e;
	}
}


HelloWorld.prototype.setPropertyGreeting = function() {
	try {
		var propertyGreeting = Config.P(NATIVE_EXTENSION_PROPERTY_SAMPLE);
		if (propertyGreeting != null) {
			this.setGreeting(propertyGreeting);
		} else {
			Util.error("No value for property name [" + NATIVE_EXTENSION_PROPERTY_SAMPLE + "] found.");
		}
		
	} catch (e) {
		Util.error("Problem setting greeting from property name [" + NATIVE_EXTENSION_PROPERTY_SAMPLE + "]: " + e.message);
		throw e;
	}
}


HelloWorld.prototype.getAppName = function() {
    return NATIVE_EXTENSION_APP_NAME;
}
 
HelloWorld.prototype.printGreeting = function() {
    var status = HelloWorld.prototype.STATUS_ERROR;

    try {
       Ti.API.error(HelloWorldModule.example());
       status = HelloWorld.prototype.STATUS_OK;
    } 
    catch (err) {
       Ti.API.error("Error printing out " + NATIVE_EXTENSION_APP_NAME + " greeting");        
    }
    
    return status;
} 

HelloWorld.prototype.cleanup = function() {
	Ti.App.removeEventListener(GET_GREETING_EVENT, this.getGreetingEventHandlerProxy);
	Ti.App.removeEventListener(SET_GREETING_EVENT, this.setGreetingEventHandlerProxy);
}

/**
 * Event handler for GET_GREETING_EVENT. Get the greeting message. Fire an event back with
 * greeting message and a 'success' status. If error, then fire back 'failed' along with the
 * error message.
 *
 * @param args: Event argument defined as:
 *   originId: ID of the event originator; if given, it's sent back in the response
 */
HelloWorld.prototype.getGreetingEventHandler = function(args) {
	args = args || {};
    Util.info('received event: ' + GET_GREETING_EVENT + ' in module: ' + JSON.stringify(args));
 
 	var resp = {
 		originId: args.originId || undefined
 	}

    try {
		resp.greeting = HelloWorldModule.example();
    	resp.status = 'success';
    } catch (e) {
		resp.status = 'failed';
		resp.message = args.message;
	}

	Util.info('firing event: ' + GET_GREETING_RESP_EVENT + ' from module to: ' + resp.originId);
	Ti.App.fireEvent(GET_GREETING_RESP_EVENT, resp);
}

/**
 * Event handler for SET_GREETING_EVENT. Set the greeting message to that given in the event arg.
 * Upon successfully setting the greeting, a 'success' response is fired back. If there's an error,
 * 'failed' is sent back along with the error message.
 *
 * @param args: Event argument defined as:
 *   originId: ID of the event originator; if given, it's sent back in response
 *   greeting: greeting message to use
 */
HelloWorld.prototype.setGreetingEventHandler = function(args) {
	args = args || {};
    Util.info('received event: ' + SET_GREETING_EVENT + ' in module: ' + JSON.stringify(args));
    
    var errorProp = Config.P(NATIVE_EXTENSION_ERROR_CODE);
    if (errorProp && errorProp !== '0') {
    	AppActivity.logActivity("com.mfoundry.mb.helloworld", "GreetingsSimulatedError", {"location" : "greetingsEventHandler"}, errorProp);
    }
 
 	var resp = {
 		originId: args.originId || undefined
 	}

    try {
    	this.setGreeting(args.greeting);
    	resp.status = 'success';
    	if (errorProp && errorProp !== '0')  {
    		resp.message = '(with simulated app activity error)';
    	}
    } catch (e) {
		resp.status = 'failed';
		resp.message = args.message;
	}

	Util.info('firing event: ' + SET_GREETING_RESP_EVENT + ' from module to: ' + resp.originId + ' status: ' + resp.status);
	Ti.App.fireEvent(SET_GREETING_RESP_EVENT, resp);
}

/**
 * Setup app-level event handlers that this module supports. A wrapper proxy receives the
 * actual event which then calls the actual handler with a corrected 'this' object.
 */
function _setupAppEventHandlers() {
    var that = this;
	this.getGreetingEventHandlerProxy = function(e) {
		that.getGreetingEventHandler(e);
	}
	this.setGreetingEventHandlerProxy = function(e) {
		that.setGreetingEventHandler(e);
	}
	Util.info('registering for event: ' + GET_GREETING_EVENT);
	Ti.App.addEventListener(GET_GREETING_EVENT, this.getGreetingEventHandlerProxy);
	Util.info('registering for event: ' + SET_GREETING_EVENT);
	Ti.App.addEventListener(SET_GREETING_EVENT, this.setGreetingEventHandlerProxy);
}

module.exports = HelloWorld;
